/* Andrey N. Kuznetsov, Almaty, 2012 *
 * For educational purposes.         *
 * Under GPL 2.0                     */
//---------------------------------------------------------------------
#include "TCentralComponent.h"
#include "TTreeViewItem.h"
//---------------------------------------------------------------------
#define tr(s) String::fromUTF8(s)
//---------------------------------------------------------------------
TCentralComponent::TCentralComponent() : Component ("Central Component"),
                                         pItemsGroup(0),
                                         pGenreTreeView(0),
                                         pNewItemButton(0),
                                         pNewSubItemButton(0),
                                         pDeleteItemButton(0),
                                         pPropertiesGroup(0),
                                         pTextLabel(0),
                                         pTextEdit(0),
                                         pEditPropertiesCheckBox(0),
                                         pApplyButton(0)
{
    pItemsGroup = new GroupComponent("ItemsGroup", tr("Элементы"));
    addAndMakeVisible(pItemsGroup);

    pGenreTreeView = new TreeView("GenreTreeView");
    pGenreTreeView->addMouseListener(this, true);
    // Устанавливаем корневой элемент (узел) списка...
    pGenreTreeView->setRootItem(new TTreeViewItem(tr("Жанры")));
    // и выделяем его
    pGenreTreeView->getRootItem()->setSelected(true, false);
    addAndMakeVisible(pGenreTreeView);

    pNewItemButton = new TextButton("NewItemButton");
    pNewItemButton->setButtonText(tr("Новый элемент"));
    pNewItemButton->addListener(this);
    addAndMakeVisible(pNewItemButton);

    pNewSubItemButton = new TextButton("NewSubItem");
    pNewSubItemButton->setButtonText(tr("Новый субэлемент"));
    pNewSubItemButton->addListener(this);
    addAndMakeVisible(pNewSubItemButton);

    pDeleteItemButton = new TextButton("pDeleteItem");
    pDeleteItemButton->setButtonText(tr("Удалить элемент"));
    pDeleteItemButton->addListener(this);
    addAndMakeVisible(pDeleteItemButton);

    pPropertiesGroup = new GroupComponent("pPropertiesGroup", tr("Свойства элемента"));
    addAndMakeVisible(pPropertiesGroup);

    pTextLabel = new Label("pTextLabel", tr("Жанр:"));
    pTextLabel->setFont(Font(15.0000f, Font::plain));
    pTextLabel->setJustificationType(Justification::centredLeft);
    pTextLabel->setEditable(false, false, false);
    pTextLabel->setColour(Label::textColourId, Colours::black);
    pTextLabel->setColour(Label::backgroundColourId, Colours::azure);
    addAndMakeVisible(pTextLabel);

    pTextEdit = new TextEditor("TextEdit");   
    pTextEdit->setMultiLine(false);
    pTextEdit->setReturnKeyStartsNewLine(false);
    pTextEdit->addKeyListener(this);
    pTextEdit->setReadOnly(false);
    pTextEdit->setScrollbarsShown(true);
    pTextEdit->setCaretVisible(true);
    pTextEdit->setPopupMenuEnabled(true);
    pTextEdit->setTextToShowWhenEmpty(tr("Введите название"), Colours::grey);
    addAndMakeVisible(pTextEdit);

    pEditPropertiesCheckBox = new ToggleButton(tr("Редактировать свойства"));
    pEditPropertiesCheckBox->addListener(this);
    addAndMakeVisible(pEditPropertiesCheckBox);
    
    pApplyButton = new TextButton("ApplyButton");
    pApplyButton->setButtonText(tr("Применить"));
    pApplyButton->setEnabled(false);
    pApplyButton->addListener(this);
    addAndMakeVisible(pApplyButton);

    setSize(710, 430);
}
//---------------------------------------------------------------------
TCentralComponent::~TCentralComponent()
{
    // Удаляем корневой узел и все дочерние элементы
    pGenreTreeView->deleteRootItem();
    // Удаляем дочерние виджеты
    // и обнуляем их указатели
    deleteAllChildren();
}
//---------------------------------------------------------------------
void TCentralComponent::paint(Graphics& Canvas)
{
    Canvas.fillAll(Colours::azure);
}
//---------------------------------------------------------------------
void TCentralComponent::resized()
{
    pItemsGroup->setBounds(10, 10, 350, 400);
    pGenreTreeView->setBounds(25, 35, 200, 355);
    pNewItemButton->setBounds(240, 35, 100, 25);
    pNewSubItemButton->setBounds(240, 75, 100, 25);
    pDeleteItemButton->setBounds(240, 115, 100, 25);
    pPropertiesGroup->setBounds(370, 10, 330, 400);
    pTextLabel->setBounds(385, 35, 70, 25);
    pTextEdit->setBounds(465, 35, 222, 25);
    pEditPropertiesCheckBox->setBounds(385, 65, 150, 25);
    pApplyButton->setBounds(585, 65, 100, 25);
}
//---------------------------------------------------------------------
void TCentralComponent::AddNewItem()
{
    // Если поле ввода не пусто...
    if(pTextEdit->getText().isNotEmpty())
    {
        // Находим корневой элемент списка и добавляем в его конец
        // дочерний элемент
        pGenreTreeView->getRootItem()->addSubItem(new TTreeViewItem(pTextEdit->getText()));
        // Находим корневой элемент списка и выделяем последний
        // из его дочерних элементов
        pGenreTreeView->getRootItem()->getSubItem(pGenreTreeView->getRootItem()->getNumSubItems() - 1)->setSelected(true, true); 
        pTextEdit->clear();
    }
}
//---------------------------------------------------------------------
void TCentralComponent::buttonClicked(Button* pButton)
{
    // Если нажата кнопка "Новый элемент"
    if(pButton == pNewItemButton)
    {
        // то добавляем его
        AddNewItem();
    }
    // Если нажата кнопка "Новый субэлемент"
    else if(pButton == pNewSubItemButton)
    {
        // и если имеется текст в поле ввода
        if(pTextEdit->getText().isNotEmpty())
        {
            // то находим выделенный элемент
            // и добавляем к нему дочерний элемент
            pGenreTreeView->getSelectedItem(0)->addSubItem(new TTreeViewItem(pTextEdit->getText()));
            // Очищаем поле ввода
            pTextEdit->clear();
        }
    }
    // Если нажата кнока "Удалить элемент"
    else if(pButton == pDeleteItemButton)
    {
        // Если выбранный элемент - корневой узел
        if(pGenreTreeView->getSelectedItem(0) == pGenreTreeView->getRootItem())
        {
            // то сообщаем пользователю о невозможности его удаления
            AlertWindow::showMessageBox(AlertWindow::WarningIcon, tr("Ошибка"), tr("Нельзя удалять корневой элемент!"), tr("Принято"), 0);
            return;
        }

        // Запоминаем индекс выделенного элемента в наборе дочерних элементов 
        // родительского узла
        int i = pGenreTreeView->getSelectedItem(0)->getIndexInParent();
        // Удаляем субэлемент, пользуясь его индексом
        pGenreTreeView->getSelectedItem(0)->getParentItem()->removeSubItem(i, true);
        // Если ещё остались дочерние элементы в корневом узле
        if(pGenreTreeView->getRootItem()->getNumSubItems() != 0)
        {
            // то выделяем последний
            pGenreTreeView->getRootItem()->getSubItem(pGenreTreeView->getRootItem()->getNumSubItems() - 1)->setSelected(true, false);
        }
        // Иначе - выделяем корень
        else pGenreTreeView->getRootItem()->setSelected(true, false);
    }
    // Если отмечен флажок "Редактировать свойства"
    else if(pButton == pEditPropertiesCheckBox)
    {
        // Делаем доступной кнопку "Применить" в зависмости от состояния флажка
        pApplyButton->setEnabled(pEditPropertiesCheckBox->getToggleState());
        // Если флажок отмечен и если выбранный элемент - не корень
        if(pEditPropertiesCheckBox->getToggleState() && pGenreTreeView->getSelectedItem(0) != pGenreTreeView->getRootItem())
        {
            // отображаем в поле ввода имя выбранного элемента 
            pTextEdit->setText(pGenreTreeView->getSelectedItem(0)->getUniqueName());
        }
        // Иначе - очищаем поле ввода
        else pTextEdit->clear();
    }
    // Если нажата кнопка "Применить"
    else if(pButton == pApplyButton)
    {
        // Если выбранный элемент - корневой узел
        if(pGenreTreeView->getRootItem() == pGenreTreeView->getSelectedItem(0)) 
        {
            // то сообщаем пользователю о невозможности редактирования его свойств
            AlertWindow::showMessageBox(AlertWindow::WarningIcon, tr("Ошибка"), tr("Нельзя изменить свойства корневого элемента!"), tr("Принято"), 0);
            return;
        }

        // Если не введено новое имя узла, выходим... 
        if(pTextEdit->getText().isEmpty()) return;
        // Назначаем новое имя выбранному элементу
        ((TTreeViewItem*)pGenreTreeView->getSelectedItem(0))->SetNewName(pTextEdit->getText());
        // Перерисовываем дерево
        this->repaint();
    }
}
//---------------------------------------------------------------------
void TCentralComponent::mouseDown(const MouseEvent& Event)
{
    if(pEditPropertiesCheckBox->getToggleState())
    {
        pTextEdit->setText(pGenreTreeView->getSelectedItem(0)->getUniqueName());
    }
}
//---------------------------------------------------------------------
bool TCentralComponent::keyPressed(const KeyPress& Key, Component* pComponent)
{
    // Если над полем ввода нажали клавишу ENTER...
    if(Key == KeyPress::returnKey && pComponent == pTextEdit)
    {
	// добавляем новый элемент в узел
        AddNewItem();
        return true;
    }
    else return false;
}
//---------------------------------------------------------------------
